@@ -5,7 +5,8 @@ from .gradient import Gradient
 from .layerobj import _LayerObj
 from .layout import Layout
 from .legenditem import LegendItem
-from .metadata import Metadata
+from .metadata import Metadata, MetaDiv
 from .technique import Technique
 from .versions import Versions
 from .helpers import *
+from .objlink import Link, LinkDiv
